import time
from typing import Any, Callable, Generator, TypeVar
from contextlib import contextmanager


T = TypeVar('T')
def retriable(
    func: Callable[..., T],
    *args,

    max_retries: int = 3,
    sleep_secs:  float = 1.0,
    validate_value: Callable[[T], bool] | None = None,
    on_try: Callable[[int, int], None] | None = None,

    raise_final_exception: bool = True,
    debug: bool = False,
    **kwargs
) -> T:
    current_retries = 0
    exception: Exception | None = None

    while current_retries < max_retries:
        if on_try:
            on_try(current_retries+1, max_retries)

        try:
            value = func(*args, **kwargs)

            if not validate_value or validate_value(value):
                return value
        except Exception as e:
            if debug:
                print(f'Exception: {e}')

            exception = e

        current_retries += 1
        if current_retries == max_retries - 1:
            break

        time.sleep(sleep_secs)

    if raise_final_exception and exception is not None:
        raise exception

    return None # type: ignore

@contextmanager
def retriable_ctm(
    func: Callable[..., T],
    *args,

    max_retries: int = 3,
    sleep_secs:  float = 1.0,
    validate_value: Callable[[T], bool] | None = None,
    on_try: Callable[[int, int], None] | None = None,

    raise_final_exception: bool = True,
    debug: bool = False,
    **kwargs
) -> Generator[T, Any, Any]:
    yield retriable(
        func,
        *args,
        max_retries=max_retries,
        sleep_secs=sleep_secs,
        validate_value=validate_value,
        on_try=on_try,
        raise_final_exception=raise_final_exception,
        debug=debug,
        **kwargs
    )



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = [line.strip() for line in fh if line.strip() and not line.startswith("#")]

setup(
    name="retriable",
    version="0.1.1",
    description="A Python library for editing images",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/kkristof200/kretriable",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Multimedia :: Graphics",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.8",
    install_requires=requirements,
    keywords="retriable",
    project_urls={
        "Homepage": "https://github.com/kkristof200/kretriable",
        "Repository": "https://github.com/kkristof200/kretriable",
        "Issues": "https://github.com/kkristof200/kretriable/issues",
    },
)